using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Kinect;
using Microsoft.Speech.Recognition;
using Microsoft.Samples.Kinect.Avateering;
using Microsoft.Samples.Kinect.BasicInteractions;

namespace MyGame
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        private SpriteBatch spriteBatch;

        List<CModel> models = new List<CModel>();
        Camera camera;

        MouseState lastMouseState;

        ObjectAnimation anim_zal1_in;
        ObjectAnimation anim_zal1_out;
        ObjectAnimation anim_zal2_in;
        ObjectAnimation anim_zal2_out;
        ObjectAnimation anim_zal3_in;
        ObjectAnimation anim_zal3_out;
        ObjectAnimation anim_zal4_in;
        ObjectAnimation anim_zal4_out;

        private SpeechRecognitionEngine _recognizer;

        int zal_num = 0;
        bool zal_zoom = false;

        Vector3 def_cam_pos = new Vector3(1200, 9000, -1400);

        Texture2D hand_r;
        Texture2D hand_l;
        Texture2D home;
        Texture2D rastr;

        private readonly KinectChooser chooser;
        private HandPosition ActiveHand;

        int width = 1920;
        int height = 1080;
        int width_dop = 960;
        int height_dop = 540;

        private bool skeletonDetected;
        private int timer;
        
        Effect effect_depth;
        Effect effect_simple;

        RenderTarget2D colorTarget;
        RenderTarget2D depthTarget;
        Texture2D ColorTarget_2DZ;
        Texture2D DepthTarget_2DZ;

        FpsMonitor FPS;
        SpriteFont font;

        private static RecognizerInfo GetKinectRecognizer()
        {
            foreach (RecognizerInfo recognizer in SpeechRecognitionEngine.InstalledRecognizers())
            {
                if ("ru-Ru".Equals(recognizer.Culture.Name, StringComparison.OrdinalIgnoreCase))
                {
                    return recognizer;
                }
            }
            return null;
        }

        void _recognizer_SpeechRecognized(object sender, SpeechRecognizedEventArgs e)
        {
            foreach (RecognizedWordUnit lWord in e.Result.Words)
            {
                switch (lWord.Text)
                {
                    case "":
                        if (zal_zoom == false)
                        {
                            zal_zoom = true;
                            zal_num = 1;
                        }
                        break;
                    case "":
                        if (zal_zoom == false)
                        {
                            zal_zoom = true;
                            zal_num = 2;
                        }
                        break;
                    case "":
                        if (zal_zoom == false)
                        {
                            zal_zoom = true;
                            zal_num = 3;
                        }
                        break;
                    case "":
                        if (zal_zoom == false)
                        {
                            zal_zoom = true;
                            zal_num = 4;
                        }
                        break;
                    case "":
                        if (zal_zoom == true)
                        {
                            zal_zoom = false;
                        }
                        break;
                }
            }
        }

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreferredBackBufferWidth = width;
            graphics.PreferredBackBufferHeight = height;
            graphics.IsFullScreen = true;

            // The Kinect sensor will use 640x480 for the color stream (default) and 320x240 for depth
            this.chooser = new KinectChooser(this, ColorImageFormat.RgbResolution640x480Fps30, DepthImageFormat.Resolution320x240Fps30);
            this.Services.AddService(typeof(KinectChooser), this.chooser);

            // Optionally set near mode for close range avateering (0.4m up to 3m)
            this.chooser.NearMode = true;

            // Optionally set seated mode for upper-body only tracking here (typically used with near mode for close to camera tracking)
            this.chooser.SeatedMode = false;

            // Adding these objects as XNA Game components enables automatic calls to the overridden LoadContent, Update, etc.. methods
            this.Components.Add(this.chooser);

            this.skeletonDetected = false;

            this.ActiveHand = new HandPosition();

            this.FPS = new FpsMonitor();

        }

        /// <summary>
        /// Gets the KinectChooser from the services.
        /// </summary>
        public KinectChooser Chooser
        {
            get
            {
                return (KinectChooser)this.Services.GetService(typeof(KinectChooser));
            }
        }

        /// <summary>
        /// Gets the SpriteBatch from the services.
        /// </summary>
        public SpriteBatch SharedSpriteBatch
        {
            get
            {
                return (SpriteBatch)this.Services.GetService(typeof(SpriteBatch));
            }
        }

        /// <summary>
        /// Gets or sets the last frames skeleton data.
        /// </summary>
        private static Skeleton[] SkeletonData { get; set; }

        protected override void Initialize()
        {

            RecognizerInfo ri = GetKinectRecognizer();
            if (null != ri)
                _recognizer = new SpeechRecognitionEngine(ri.Id);
            else
                _recognizer = new SpeechRecognitionEngine();

            _recognizer.SetInputToDefaultAudioDevice();

            Choices lChoices = new Choices("", "", "", "", "");
            GrammarBuilder lGrammarBuilder = new GrammarBuilder(lChoices);
            Grammar lGrammar = new Grammar(lGrammarBuilder);

            lGrammarBuilder.Culture = System.Globalization.CultureInfo.CurrentCulture;
            _recognizer.LoadGrammar(lGrammar);

            _recognizer.SpeechRecognized += new EventHandler<SpeechRecognizedEventArgs>(_recognizer_SpeechRecognized);
            _recognizer.RecognizeAsync(RecognizeMode.Multiple);

            colorTarget = new RenderTarget2D(GraphicsDevice, this.width_dop, this.height_dop);
            depthTarget = new RenderTarget2D(GraphicsDevice, this.width_dop, this.height_dop);

            base.Initialize();
        }

        // Called when the game should load its content
        protected override void LoadContent()
        {
            // Create the spritebatch to draw the 3D items
            this.spriteBatch = new SpriteBatch(this.GraphicsDevice);
            this.Services.AddService(typeof(SpriteBatch), this.spriteBatch);

            //spriteBatch = new SpriteBatch(GraphicsDevice);

            hand_l = Content.Load<Texture2D>("hand_l");
            hand_r = Content.Load<Texture2D>("hand_r");
            home = Content.Load<Texture2D>("home");
            rastr = Content.Load<Texture2D>("rastr4");

            models.Add(new CModel(Content.Load<Model>("room"),
                Vector3.Zero, Vector3.Zero, new Vector3(140), GraphicsDevice));

            models.Add(new CModel(Content.Load<Model>("teapot"),
                new Vector3(-200,0,300), Vector3.Zero, new Vector3(50),
                GraphicsDevice));

            models.Add(new CModel(Content.Load<Model>("teapot"),
                new Vector3(2400, 0, -2400), Vector3.Zero, new Vector3(50),
                GraphicsDevice));

            models.Add(new CModel(Content.Load<Model>("teapot"),
                new Vector3(0, 0, -3000), Vector3.Zero, new Vector3(50),
                GraphicsDevice));

            models.Add(new CModel(Content.Load<Model>("teapot"),
                new Vector3(2200, 0, 600), Vector3.Zero, new Vector3(50),
                GraphicsDevice));

            effect_depth = Content.Load<Effect>("ShadowDepthEffect");
            effect_simple = Content.Load<Effect>("SimpleEffect");

            camera = new FreeCamera(def_cam_pos,
                MathHelper.ToRadians(0),
                MathHelper.ToRadians(-90),
                GraphicsDevice);

            this.font = Content.Load<SpriteFont>("Segoe16");

            reset_in_animation();
            reset_out_animation();
            
            lastMouseState = Mouse.GetState();

        }

        void reset_in_animation()
        {
            anim_zal1_in = new ObjectAnimation(def_cam_pos, new Vector3(-200, 6000, 300), Vector3.Zero,
                    new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
            anim_zal2_in = new ObjectAnimation(def_cam_pos, new Vector3(2400, 6000, -2400), Vector3.Zero,
                    new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
            anim_zal3_in = new ObjectAnimation(def_cam_pos, new Vector3(0, 6000, -3000), Vector3.Zero,
                    new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
            anim_zal4_in = new ObjectAnimation(def_cam_pos, new Vector3(2200, 6000, 600), Vector3.Zero,
                    new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
        }

        void reset_out_animation()
        {
            anim_zal1_out = new ObjectAnimation(new Vector3(-200, 6000, 300), def_cam_pos, Vector3.Zero,
                new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
            anim_zal2_out = new ObjectAnimation(new Vector3(2400, 6000, -2400), def_cam_pos, Vector3.Zero,
                    new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
            anim_zal3_out = new ObjectAnimation(new Vector3(0, 6000, -3000), def_cam_pos, Vector3.Zero,
                    new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
            anim_zal4_out = new ObjectAnimation(new Vector3(2200, 6000, 600), def_cam_pos, Vector3.Zero,
                    new Vector3(0, -MathHelper.TwoPi, 0), TimeSpan.FromSeconds(3), false);
        }

        // Called when the game should update itself
        protected override void Update(GameTime gameTime)
        {
            updateCamera(gameTime);

            if (zal_zoom == true)
            {
                reset_out_animation();
                if (zal_num == 1)
                {
                    anim_zal1_in.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal1_in.Position;
                    ((FreeCamera)camera).Update();
                }
                if (zal_num == 2)
                {
                    anim_zal2_in.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal2_in.Position;
                    ((FreeCamera)camera).Update();
                }
                if (zal_num == 3)
                {
                    anim_zal3_in.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal3_in.Position;
                    ((FreeCamera)camera).Update();
                }
                if (zal_num == 4)
                {
                    anim_zal4_in.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal4_in.Position;
                    ((FreeCamera)camera).Update();
                }
            }
            else
            {
                reset_in_animation();
                if (zal_num == 1)
                {
                    anim_zal1_out.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal1_out.Position;
                    ((FreeCamera)camera).Update();
                }
                if (zal_num == 2)
                {
                    anim_zal2_out.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal2_out.Position;
                    ((FreeCamera)camera).Update();
                }
                if (zal_num == 3)
                {
                    anim_zal3_out.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal3_out.Position;
                    ((FreeCamera)camera).Update();
                }
                if (zal_num == 4)
                {
                    anim_zal4_out.Update(gameTime.ElapsedGameTime);
                    ((FreeCamera)camera).Position = anim_zal4_out.Position;
                    ((FreeCamera)camera).Update();
                }
            }

            // If the sensor is not found, not running, or not connected, stop now
            if (null != this.chooser && null != this.Chooser.Sensor && false != this.Chooser.Sensor.IsRunning && this.Chooser.Sensor.Status == KinectStatus.Connected)
            {
                using (var skeletonFrame = this.Chooser.Sensor.SkeletonStream.OpenNextFrame(0))
                {
                    // Sometimes we get a null frame back if no data is ready
                    if (null != skeletonFrame)
                    {
                        // Reallocate if necessary
                        if (null == SkeletonData || SkeletonData.Length != skeletonFrame.SkeletonArrayLength)
                        {
                            SkeletonData = new Skeleton[skeletonFrame.SkeletonArrayLength];
                        }

                        skeletonFrame.CopySkeletonDataTo(SkeletonData);

                        //skeletonFrame.CopySkeletonDataTo(skeletons);

                        // Select the first tracked skeleton we see to avateer
                        Skeleton rawSkeleton =
                            (from s in SkeletonData
                             where s != null && s.TrackingState == SkeletonTrackingState.Tracked
                             select s).FirstOrDefault();

                        if (null != rawSkeleton)
                        {
                            this.skeletonDetected = true;
                        }
                        else
                        {
                            this.skeletonDetected = false;
                        }

                        int playerIndex = 0;
                        Skeleton updatedSkeleton = null;
                        updatedSkeleton = SkeletonData[playerIndex];
                        if (updatedSkeleton != null)
                        {
                            SkeletonUpdated(updatedSkeleton);
                            if (this.ActiveHand != null)
                                CheckPosition(gameTime);
                        }
                    }
                }
            }

            FPS.Update();

            base.Update(gameTime);
 
        }

        void CheckPosition(GameTime gameTime)
        {
            if (zal_zoom == false)
            {
                if (this.ActiveHand.X > 400 && this.ActiveHand.X < 800 && this.ActiveHand.Y > 200 && this.ActiveHand.Y < 500)
                {
                    this.timer += gameTime.ElapsedGameTime.Milliseconds;
                    if (this.timer > 2000)
                    {
                        zal_zoom = true;
                        zal_num = 3;
                    }
                }
                else if (this.ActiveHand.X > 400 && this.ActiveHand.X < 800 && this.ActiveHand.Y > 500 && this.ActiveHand.Y < 800)
                {
                    this.timer += gameTime.ElapsedGameTime.Milliseconds;
                    if (this.timer > 2000)
                    {
                        zal_zoom = true;
                        zal_num = 1;
                    }
                }
                else if (this.ActiveHand.X > 800 && this.ActiveHand.X < 1200 && this.ActiveHand.Y > 200 && this.ActiveHand.Y < 500)
                {
                    this.timer += gameTime.ElapsedGameTime.Milliseconds;
                    if (this.timer > 2000)
                    {
                        zal_zoom = true;
                        zal_num = 2;
                    }
                }
                else if (this.ActiveHand.X > 800 && this.ActiveHand.X < 1200 && this.ActiveHand.Y > 500 && this.ActiveHand.Y < 800)
                {
                    this.timer += gameTime.ElapsedGameTime.Milliseconds;
                    if (this.timer > 2000)
                    {
                        zal_zoom = true;
                        zal_num = 4;
                    }
                }
                else
                {
                    this.timer = 0;
                }
            }
            else
            {
                if (this.ActiveHand.X > 10 && this.ActiveHand.X < 60 && this.ActiveHand.Y > 10 && this.ActiveHand.Y < 60)
                {
                    this.timer += gameTime.ElapsedGameTime.Milliseconds; 
                    zal_zoom = false;
                }
            }
        }

        void updateCamera(GameTime gameTime)
        {
            // Get the new keyboard and mouse state
            MouseState mouseState = Mouse.GetState();
            KeyboardState keyState = Keyboard.GetState();

            // Determine how much the camera should turn
            float deltaX = (float)lastMouseState.X - (float)mouseState.X;
            float deltaY = (float)lastMouseState.Y - (float)mouseState.Y;

            // Rotate the camera
            //((FreeCamera)camera).Rotate(deltaX * .005f, deltaY * .005f);

            Vector3 translation = Vector3.Zero;

            // Determine in which direction to move the camera
            if (keyState.IsKeyDown(Keys.W)) translation += Vector3.Forward;
            if (keyState.IsKeyDown(Keys.S)) translation += Vector3.Backward;
            if (keyState.IsKeyDown(Keys.A)) translation += Vector3.Left;
            if (keyState.IsKeyDown(Keys.D)) translation += Vector3.Right;

            // Move 3 units per millisecond, independent of frame rate
            translation *= 4 * 
                (float)gameTime.ElapsedGameTime.TotalMilliseconds;

            // Move the camera
            ((FreeCamera)camera).Move(translation);

            // Update the camera
            camera.Update();

            // Update the mouse state
            lastMouseState = mouseState;
        }

        private void SkeletonUpdated(Skeleton skeleton)
        {
            if (this.ActiveHand == null)
            {
                this.ActiveHand = new HandPosition();
            }

            int leftX, leftY, rightX, rightY;
            Joint leftHandJoint = skeleton.Joints[JointType.HandLeft];
            Joint rightHandJoint = skeleton.Joints[JointType.HandRight];

            float leftZ = leftHandJoint.Position.Z;
            float rightZ = rightHandJoint.Position.Z;

            ScaleXY(skeleton.Joints[JointType.ShoulderCenter], false, leftHandJoint, out leftX, out leftY);
            ScaleXY(skeleton.Joints[JointType.ShoulderCenter], true, rightHandJoint, out rightX, out rightY);

            if (leftHandJoint.TrackingState == JointTrackingState.Tracked && leftZ < rightZ && leftY < height)
            {
                this.ActiveHand.IsLeft = true;
                this.ActiveHand.X = leftX;
                this.ActiveHand.Y = leftY;
            }
            else if (rightHandJoint.TrackingState == JointTrackingState.Tracked && rightY < height)
            {
                this.ActiveHand.IsLeft = false;
                this.ActiveHand.X = rightX;
                this.ActiveHand.Y = rightY;
            }
            else
            {
                if (this.ActiveHand != null)
                {
                    this.ActiveHand.X = -1;
                    this.ActiveHand.Y = -1;
                }

                this.ActiveHand = null;
            }

            if (this.ActiveHand == null)
            {
                return;
            }
        }

        private double ScaleY(Joint joint)
        {
            double y = ((this.height / 0.4) * -joint.Position.Y) +
                       (this.height / 2);
            return y;
        }

        private void ScaleXY(Joint shoulderCenter, bool rightHand, Joint joint, out int scaledX, out int scaledY)
        {
            double screenWidth = this.width;

            double x = 0;
            double y = ScaleY(joint);

            // if rightHand then place shouldCenter on left of screen
            // else place shouldCenter on right of screen
            if (rightHand)
            {
                x = (joint.Position.X - shoulderCenter.Position.X) * screenWidth * 2;
            }
            else
            {
                x = screenWidth - ((shoulderCenter.Position.X - joint.Position.X) * (screenWidth * 2));
            }


            if (x < 0)
            {
                x = 0;
            }
            else if (x > screenWidth - 5)
            {
                x = screenWidth - 5;
            }

            if (y < 0)
            {
                y = 0;
            }

            scaledX = (int)x;
            scaledY = (int)y;
        }

        // Called when the game should draw itself
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.BlendState = BlendState.Opaque;

            GraphicsDevice.BlendState = BlendState.Opaque;
            GraphicsDevice.SetRenderTarget(colorTarget);
            GraphicsDevice.Clear(Color.Black);
            foreach (CModel model in models)
                if (camera.BoundingVolumeIsInView(model.BoundingSphere))
                {
                    model.SetModelEffect(effect_simple, true);
                    model.Draw(camera.View, camera.Projection, ((FreeCamera)camera).Position);
                }
            GraphicsDevice.SetRenderTarget(null);

            GraphicsDevice.BlendState = BlendState.Opaque;
            GraphicsDevice.SetRenderTarget(depthTarget);
            GraphicsDevice.Clear(Color.Black);
            foreach (CModel model in models)
                if (camera.BoundingVolumeIsInView(model.BoundingSphere))
                {
                    model.SetModelEffect(effect_depth, false);
                    model.Draw(camera.View, camera.Projection, ((FreeCamera)camera).Position);
                }
            GraphicsDevice.SetRenderTarget(null);

            GraphicsDevice.Clear(Color.Black);

            Draw2DZ();

            this.SharedSpriteBatch.Begin();
            if (zal_zoom == true)
            {
                this.SharedSpriteBatch.Draw(home, new Rectangle(10, 10, 30, 60), null, Color.White);
                this.SharedSpriteBatch.Draw(home, new Rectangle(970, 10, 30, 60), null, Color.White);
            }
            if (this.skeletonDetected == true)
                if (this.ActiveHand != null)
                {
                    if (this.ActiveHand.IsLeft == true)
                    {
                        this.SharedSpriteBatch.Draw(hand_l, new Rectangle((ActiveHand.X - 30) / 2, ActiveHand.Y - 30, 30, 60), Color.White);
                        this.SharedSpriteBatch.Draw(hand_l, new Rectangle((ActiveHand.X - 30) / 2 + this.width_dop, ActiveHand.Y - 30, 30, 60), Color.White);
                    }
                    else
                    {
                        this.SharedSpriteBatch.Draw(hand_r, new Rectangle((ActiveHand.X - 30) / 2, ActiveHand.Y - 30, 30, 60), Color.White);
                        this.SharedSpriteBatch.Draw(hand_r, new Rectangle((ActiveHand.X - 30) / 2 + this.width_dop, ActiveHand.Y - 30, 30, 60), Color.White);
                    }
                }
            this.SharedSpriteBatch.Draw(rastr, new Rectangle(0, 0, this.width_dop, 1), Color.White);
            this.SharedSpriteBatch.End();

            this.SharedSpriteBatch.Begin();
            FPS.Draw(this.SharedSpriteBatch, font, new Vector2(100, 20), Color.White);
            FPS.Draw(this.SharedSpriteBatch, font, new Vector2(100 + this.width_dop, 20), Color.White);
            this.SharedSpriteBatch.End();

            base.Draw(gameTime);
        }

        void Draw2DZ()
        {
            if (ReferenceEquals(ColorTarget_2DZ, null))
            {
                ColorTarget_2DZ = new Texture2D(GraphicsDevice, this.width_dop, this.height, false, SurfaceFormat.Color);
            }
            if (ReferenceEquals(DepthTarget_2DZ, null))
            {
                DepthTarget_2DZ = new Texture2D(GraphicsDevice, this.width_dop, this.height, false, SurfaceFormat.Color);
            }

            var Buffer_color = new uint[this.width_dop * this.height_dop];
            var Buffer_depth = new uint[this.width_dop * this.height_dop];
            var Buffer_color2 = new uint[this.width_dop * this.height_dop*2];
            var Buffer_depth2 = new uint[this.width_dop * this.height_dop*2];

            colorTarget.GetData<uint>(Buffer_color);
            depthTarget.GetData<uint>(Buffer_depth);

            for (int y = 0; y < this.height_dop; y++)
            {
                for (int x = 0; x < this.width_dop; x++)
                {
                    Buffer_color2[y * this.width_dop * 2 + x] = Buffer_color[y * this.width_dop + x];
                    Buffer_depth2[y * this.width_dop * 2 + x] = Buffer_depth[y * this.width_dop + x];
                }
            }

            //GraphicsDevice.Textures[0] = null;
            ColorTarget_2DZ.SetData<uint>(Buffer_color2);
            DepthTarget_2DZ.SetData<uint>(Buffer_depth2);

            this.SharedSpriteBatch.Begin();
            /*
            this.SharedSpriteBatch.Draw(colorTarget, new Rectangle(graphics.GraphicsDevice.Viewport.X, graphics.GraphicsDevice.Viewport.Y,
                graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Height), Color.White);
            this.SharedSpriteBatch.Draw(depthTarget, new Rectangle(graphics.GraphicsDevice.Viewport.X + graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Y,
                graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Height), Color.White);
             */ 
            //this.SharedSpriteBatch.Draw(Target_2DZ, new Rectangle(graphics.GraphicsDevice.Viewport.X, graphics.GraphicsDevice.Viewport.Y,
                //graphics.GraphicsDevice.Viewport.Width, graphics.GraphicsDevice.Viewport.Height), Color.White);
            this.SharedSpriteBatch.Draw(ColorTarget_2DZ, new Rectangle(graphics.GraphicsDevice.Viewport.X, graphics.GraphicsDevice.Viewport.Y,
                graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Height), Color.White);
            this.SharedSpriteBatch.Draw(DepthTarget_2DZ, new Rectangle(graphics.GraphicsDevice.Viewport.X + graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Y,
                graphics.GraphicsDevice.Viewport.Width / 2, graphics.GraphicsDevice.Viewport.Height), Color.White);
            this.SharedSpriteBatch.End();
        }
    }
}



